#include <StorageKit.h>
#include <Application.h>
#include <Roster.h>
#include <String.h>

#include <InterfaceKit.h>

#include "attr.h" // attribute handling, signatures

#define BES B_EMPTY_STRING

const char *tarts_sig = "application/x-TARTS";
const char *tarts_config_sig = "application/x-TARTS_config";

enum { MSG_TYPE = 1000, MSG_COMMAND, MSG_ABOUT, MSG_CANCEL, MSG_REVERT, MSG_SAVE };

int32 width =  320;
int32 height = 120;
int32 x =      200;
int32 y =      100;

class Window1 : public BWindow {
public:
  Window1(const char* _path);
  void MessageReceived(BMessage* m);
private:
  void Revert(void);
  void Update(void);
  void Enable(void);
  void SetType(void);
  void SetCommand(void);
  void SetLabel(void);
  void Dropped(BMessage* m);
  void GetSig(const char *p);
     
  TARTS_attr *ta;
  BPath path;
  
  BBox *bbox;
  BPopUpMenu *menu; BMenuField *menufield;
  BStringView *s_command; BTextControl *t_command;          
  BButton *b_about, *b_cancel, *b_revert, *b_save;
	
  int32 n_type, o_type, type_amount;
  BString n_command, o_command;
  BString* type_description;
  bool modified;
};


// ---------------------  constructor  -------------------------------------------
Window1::Window1(const char *_path) : BWindow(BRect(x, y, width, height),
	"TARTS config", B_TITLED_WINDOW, B_NOT_ZOOMABLE | B_QUIT_ON_WINDOW_CLOSE)
{
  path.SetTo(_path);
// read TARTS:type and TARTS:command
  ta = new TARTS_attr(_path);
  o_type = ta->GetType();
  o_command = ta->GetCommand();

// initialise type descriptions
  type_amount = 8;
  type_description = new BString[type_amount] = {
  "0 - run in terminal (uses script), delay closing"
, "1 - run in terminal (uses script)"
, "2 - run in background (uses script)"
, "3 - just run it"
, "4 - launch as BeOS Application"
, "5 - launch as BeOS App (message after launch)"
, "6 - launch (signature) as BeOS App"
, "7 - launch (sig) as BeOS App (message after launch)"};

// modify window
  SetSizeLimits(width, 4000, height, 0);
  ResizeTo(width, height);  

// type choice menu
  menu = new BPopUpMenu("_menu");  
  int32 i;
  for (i = 0; i < type_amount; i++) {
    BMenuItem *mi = new BMenuItem(type_description[i].String(), new BMessage(MSG_TYPE), 0, 0);
    menu->AddItem(mi);
  }

  menufield = new BMenuField(BRect(8, 8, width - 8, 28), "_menufield",
    "Type:", menu, false, //bool fixedSize,
    B_FOLLOW_LEFT, B_WILL_DRAW|B_NAVIGABLE);
  menufield->SetDivider(40.0);
  menufield->ResizeToPreferred();


// label over textcontrol
  s_command = new BStringView(BRect(12, 48, 12, 56), BES, BES); 

// textcontrol (enter command here)
  t_command = new BTextControl(BRect(8, 64, width - 8, 72),
    "t_command", BES, BES,
    new BMessage(MSG_COMMAND), B_FOLLOW_LEFT_RIGHT, B_WILL_DRAW | B_NAVIGABLE);
  t_command->TextView()->SetMaxBytes(4096); // max size allowed for attributes
  t_command->SetDivider(0.0);

  // "About (?)" button.
  b_about = new BButton(BRect(8, 88, 24, 112), 
          "b_about", "?", new BMessage(MSG_ABOUT), B_FOLLOW_LEFT);

  // "Cancel" button.
  b_cancel = new BButton(BRect(32, 88, 98, 112), 
          "b_cancel", "Cancel", new BMessage(MSG_CANCEL), B_FOLLOW_LEFT);

  // "Revert" button.
  b_revert = new BButton(BRect(106, 88, 172, 112), 
          "b_revert", "Revert", new BMessage(MSG_REVERT), B_FOLLOW_LEFT);

  // "Save" button.
  b_save = new BButton(BRect(width - 72, 88, width - 8, 112),
          "b_save", "Save", new BMessage(MSG_SAVE), B_FOLLOW_RIGHT);          
  
    
  // Create the background box.
  bbox = new BBox(BRect(0, 0, width, height), BES, B_FOLLOW_ALL, 
          B_WILL_DRAW | B_FRAME_EVENTS | B_NAVIGABLE_JUMP, 
          B_PLAIN_BORDER); 

  AddChild(bbox);
  bbox->AddChild(menufield); bbox->AddChild(s_command); bbox->AddChild(t_command);
  bbox->AddChild(b_about); bbox->AddChild(b_cancel); bbox->AddChild(b_revert); bbox->AddChild(b_save);

  Revert(); // initialise unmodified state
} 
//--------------  end of constructor  ---------------------------

void Window1::Revert(void) {
  n_type = o_type; SetType();
  n_command = o_command; SetCommand();
  SetLabel(); // label (includes path of TARTS add-on)
  modified = false;
  Enable();
}

void Window1::Update(void) {
  modified = ((n_type != o_type) || (n_command != o_command));
  Enable();
}

void Window1::Enable(void) {
  b_save->SetEnabled(modified);
  b_revert->SetEnabled(modified);
  UpdateIfNeeded();
}

void Window1::SetCommand(void) {
  t_command->SetText(n_command.String());
}

void Window1::SetType(void) {
  BMenuItem *mi = menu->FindItem(type_description[n_type].String());
  if (mi) mi->SetMarked(true);
}

void Window1::SetLabel(void) {
  BString label("Command to run from "); label << path.Leaf();
  s_command->SetText(label.String()); 
  s_command->ResizeToPreferred();
}
          
// -----------  MessageRecieved  ----------
void Window1::MessageReceived(BMessage* m) {
  // Figure out what kind of message was received.
  switch(m->what) {
  
    // type has been changed
    case MSG_TYPE: { int32 i = 0; m->FindInt32("index", &i); n_type = i;
      Update(); break; }
    
    // Command has been updated
    case MSG_COMMAND: { n_command = t_command->Text(); Update(); break; }

    // About
    case MSG_ABOUT: { be_app->PostMessage(B_ABOUT_REQUESTED); break; }

    // Cancel => Quit
    case MSG_CANCEL: { be_app->PostMessage(B_QUIT_REQUESTED); break; }

	// Revert to previous state
    case MSG_REVERT: { Revert(); break; }
    
    // Save and quit (write TARTS:type and TARTS:command)
    case MSG_SAVE: { ta->SetValues(n_type, n_command); ta->Write();
      be_app->PostMessage(B_QUIT_REQUESTED); break; }

    // drag and drop
    case B_SIMPLE_DATA: { Dropped(m); break; }
    
    // Default: let the original BWindow handle the message
    default: { BWindow::MessageReceived(m); break; }
  }
}

void Window1::Dropped(BMessage* m) {
  entry_ref a_ref; BEntry entry; BPath p;
  m->FindRef("refs", 0, &a_ref);
  entry.SetTo(&a_ref); entry.GetPath(&p); 

  if (n_type < 6) n_command = p.Path(); 
  else GetSig(p.Path());
  t_command->SetText(n_command.String());
  Update();
}

void Window1::GetSig(const char *p) {
  BFile file(p, B_READ_ONLY);
  attr_info ati;
  if (B_OK == file.GetAttrInfo("BEOS:APP_SIG", &ati)) {
    char* buf = new char[ati.size];
    file.ReadAttr("BEOS:APP_SIG", B_STRING_TYPE, 0, buf, ati.size);
    n_command = buf;
  }
}


// ---------------------  The Application class  ---------------------------

class App : public BApplication {
  public:
	App (void);
	virtual ~App (void) {}
    virtual void AboutRequested(void);
    virtual void ArgvReceived(int32 argc, char **argv);
    virtual void RefsReceived(BMessage *m);
	void ReadyToRun (void); 
  private:
    Window1 *window;
    bool argvrecieved, refsrecieved;
};

void App::AboutRequested(void) {
  BString text("TARTS & TARTS_config  (v0.1  24-Nov-2003)
Created by Marcus Petersson  <mp_1@home.se>

In the TARTS_config window, you can see the current settings for this TARTS:

'Command' is the command that this TARTS should run, using the selected files as arguments. Drag and drop an app here or just type in some shell command.

'Type' defines how to run the command. Options 0 to 3 will run the command from the shell (all except 3 use temporary scripts in /tmp/TARTS_scripts/). The first two run the script in the terminal, while the latter two run in the background.

As work directory, the script uses the Tracker's current dir (the dir containing the files you select, if any). The file arguments are passed with absolute paths.

Options 4 to 7 will instead launch the command as a BeOS Application. Options 4 and 5 require the full path of your application, while 6 and 7 require the app signature (open with the FileType add-on to see it).

Options 4 and 6 will send the arguments while the application is starting, which works best for ShowImage or MediaPlayer. Options 5 and 7 will send the arguments when the application has started up, which works best for PecoRename or Tracker.
  
If one option does not seem to work, try another.

Drop TARTS in /boot/home/config/add-ons/Tracker to install.\n\n");

  BAlert *al = new BAlert("TARTS", text.String() , "Loving it already");

  BTextView *t = al->TextView(); t->SetStylable(true);
  text_run_array *ra = t->RunArray(0,0);
  rgb_color col = {0,0,200,100}; ra->runs[0].color = col;
  BFont f(be_bold_font); f.SetSize(16.0); ra->runs[0].font = f;
  t->SetRunArray(0, 20, ra);

  al->ResizeTo(460, 400); al->Go(NULL);
}

void App::ArgvReceived(int32 argc, char **argv) {
  if (argc > 1) {
    window = new Window1(argv[1]); // first passed argument (argv[0] is self)
    argvrecieved = true; }
  else (new BAlert("TARTS_config", "TARTS_config needs an argument", "Ah, I see"))->Go();
}

void App::RefsReceived(BMessage *m) {
  entry_ref a_ref;
  if (m->FindRef("refs", 0, &a_ref) == B_NO_ERROR) {
  	BEntry entry(&a_ref); BPath path(&entry);
  	window = new Window1(path.Path());
    refsrecieved = true; }
  else (new BAlert("TARTS_config", "TARTS_config needs an argument (message)", "Ah, I see"))->Go();  
}


void App::ReadyToRun(void) {
  if (argvrecieved || refsrecieved) window->Show();
  else PostMessage(B_QUIT_REQUESTED);
}

App::App() : BApplication (tarts_config_sig) {
  argvrecieved = refsrecieved = false;
}

int main() {
  App *app = new App();
  app->Run();
  delete app; 
  return 0;
}
