#include <stdlib.h>

#include <TrackerAddOn.h>	// Protocol for the process_refs() hook function.
#include <StorageKit.h>
#include <Message.h>
#include <Alert.h>
#include <Application.h>
#include <Roster.h>
#include <String.h>
#include <unistd.h>

#include "attr.h" // attribute handling, signatures

extern "C" int32 dummy = 0; // used (in get_image_symbol) to find correct image 

const char *tarts_sig = "application/x-TARTS";
const char *tarts_config_sig = "application/x-TARTS_config";

const char *tmp_dir_name = "/tmp/TARTS_scripts/";
const char *terminal = "/boot/beos/apps/Terminal "; // keep the space after
const char *script_start = "#!/bin/sh\n"; // required by terminal to find shell


BString MakeTempFile(const BString script) {
  // create directory for scripts
  create_directory(tmp_dir_name, 0711);
  BDirectory *tmp_dir = new BDirectory(tmp_dir_name);
  // read tmp_file name from tmp_dir, then write it back (incremented)
  uint32 f_nr = 0;
  while (tmp_dir->Lock() == B_BUSY);
  tmp_dir->ReadAttr("TARTS:filename", B_UINT32_TYPE, 0, &f_nr, 4);
  f_nr++;
  tmp_dir->WriteAttr("TARTS:filename", B_UINT32_TYPE, 0, &f_nr, 4);
  tmp_dir->Unlock();
  BString tmp_name; tmp_name << (f_nr - 1);
  // write script to file and return path
  BFile tmp_file;
  tmp_dir->CreateFile(tmp_name.String(), &tmp_file);	
  tmp_file.Write(script.String(), script.Length());	
  BPath tmp_path (tmp_dir, tmp_name.String());
  return tmp_path.Path();
}

// ===========================================================================
// process_refs
// ===========================================================================

// The Tracker calls this function when the user invokes the add-on.
// The current directory is found in dir_ref. 
// msg is a standard B_REFS_RECEIVED BMessage with the "refs" array 
// containing the entry_refs of the files selected by the user.
// The third argument is currently unused.

void process_refs(entry_ref dir_ref, BMessage* msg, void*) {
  // TARTS needs to find itself
  // Get the image_info for every image in this team
  // (the app that started this add-on, typically Tracker)
  image_info ii;
  int32 cookie = 0, *_dummy;
  while (get_next_image_info(0, &cookie, &ii) == B_OK){
	if (ii.type == B_ADD_ON_IMAGE) {
      get_image_symbol(ii.id, "dummy", B_SYMBOL_TYPE_DATA, (void**)&_dummy);
      if (_dummy == &dummy) break; //break if we find &dummy
    } }

  // read attributes
  TARTS_attr *ta = new TARTS_attr((const char*)ii.name);

  int _type = ta->GetType(); // get "TARTS:type"
  BString _command = ta->GetCommand(); // get "TARTS:command" 

  if (_type < 4) {
    BString the_line, script(script_start); // create script
    BEntry entry(&dir_ref); BPath path(&entry);
    script << "cd " << path.Path() << "\n"; // cd to directory (default)
    script << _command; // insert command

  // insert all the arguments into the script
    int refs; entry_ref a_ref;
    for (refs=0; msg->FindRef("refs", refs, &a_ref) == B_NO_ERROR; refs++) {
      entry.SetTo(&a_ref); entry.GetPath(&path); 
      script << " \"" << path.Path() << "\""; } // quote argument
    script << "\n";

    switch(_type) {
	  	// 0 - run in terminal (uses script), delay closing
      case 0: { script << "read -p \"(Press Enter to close)\"\n";
  			the_line << terminal << MakeTempFile((const BString)script); break; }
	  	// 1 - run in terminal (uses script)
  	  case 1: the_line << terminal << MakeTempFile((const BString)script); break;
	  	// 2 - run in background (uses script)
  	  case 2: the_line << MakeTempFile((const BString)script); break;
  		// 3 - just run it
  	  case 3: the_line = script; break;
    }
  // run the_line
    system(the_line.String());
  // TODO: perhaps add more script options (use relative path, run in program dir)

  } else {
    bool send_after=false, command_is_sig=false;
    switch(_type) {
  		// 4 - launch as Be App
   	  case 4: break;
  		// 5 - launch as Be App (message after launch)
   	  case 5: send_after = true; break;
  		// 6 - launch (signature) as Be App
   	  case 6: command_is_sig = true; break;
  		// 7 - launch (signature) as Be App (message after launch)
   	  case 7: send_after = true; command_is_sig = true; break;
    }
  
    team_id i;  entry_ref er;
    if (command_is_sig) be_roster->FindApp(_command.String(), &er);
    else { BEntry entry(_command.String()); entry.GetRef(&er); }

    if (send_after) {
      be_roster->Launch(&er, (BMessage*)NULL, &i);
      BMessenger mess(NULL, i, NULL);
      mess.SendMessage(msg); }
    else be_roster->Launch(&er, msg);
  } // if (_type < 4)
} // void process_refs(...)

// ===========================================================================
// main
// ===========================================================================
// An add-on is not required to have a main() function. But if it does, 
// the user can run the add-on as a stand-alone application as well.
// In this case, we use it to configure the add-on.

class App : public BApplication {
  public:
	App (void);
	virtual ~App (void) {}
	void ReadyToRun (void) {PostMessage( B_QUIT_REQUESTED ); }
};

App::App() : BApplication (tarts_sig) {
  app_info ai; GetAppInfo(&ai);
  BEntry entry(&ai.ref); BPath path(&entry);
  char* argv[1] = { (char*)path.Path() };
  be_roster->Launch(tarts_config_sig, 1, argv);
}

int main() {
  App *app = new App();
  app->Run();
  delete app; 
  return 0;
}
